@@ -9,16 +9,22 @@ import random
 from hdfs.client import Client
 from hdfs.util import HdfsError
 import traceback
+from glob import glob
 
 
 # 缓存seed文件，来加速后面的随机函数random_from_seed工作
-seed_cache = {"10s": [], "100s": [], "1Ks": [], "10Ks": [], "100Ks": [], "10n": [], "100n": [], "1Kn": [], "10Kn": []}
+seed_cache = {}
 # 全局内存文件系统句柄
 g_MemoryFSHandler = None
 
 
 # 创建seed文件，默认在SQLCLI_HOME/data下
-def Create_SeedCacheFile(p_szSeedName, p_nNullValueCount=0):
+def Create_SeedCacheFile(
+        p_szDataType,                      # 种子文件的数据类型，目前支持String和integer
+        p_nDataLength,                     # 每个随机数的最大数据长度
+        p_nRows,                           # 种子文件的行数，即随机数的数量
+        p_szSeedName,
+        p_nNullValueCount=0):
     if "SQLCLI_HOME" not in os.environ:
         raise SQLCliException("Missed SQLCLI_HOME, please set it first. Seed file will created in SQLCLI_HOME/data")
 
@@ -27,114 +33,35 @@ def Create_SeedCacheFile(p_szSeedName, p_nNullValueCount=0):
     if not os.path.exists(datadir):
         os.makedirs(datadir)
 
-    if p_szSeedName not in seed_cache.keys():
-        raise SQLCliException("Unknown seed name [" + p_szSeedName + "]")
+    # 检查输入的参数
+    if p_szDataType.upper() not in ('STRING', 'INTEGER'):
+        raise SQLCliException("Data Type must be String or integer.")
+    if p_nDataLength <= 0:
+        raise SQLCliException("Data Length must great then zero.")
+    if p_nRows <= 0:
+        raise SQLCliException("rows must great then zero.")
 
-    if p_szSeedName == "10s":
+    if p_szDataType.upper() == "STRING":
         buf = []
-        for n in range(0, 10):
-            buf.append(random_ascii_letters_and_digits([100, ]) + "\n")
+        for n in range(0, p_nRows):
+            buf.append(random_ascii_letters_and_digits([p_nDataLength, ]) + "\n")
         if p_nNullValueCount > 0:
             for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 10 - 1)
+                m_nPos = random.randint(0, p_nRows - 1)
                 buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.10")
+        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", p_szSeedName + ".seed")
         with open(seed_file, 'w') as f:
             f.writelines(buf)
 
-    if p_szSeedName == "100s":
+    if p_szDataType.upper() == "INTEGER":
         buf = []
-        for n in range(0, 100):
-            buf.append(random_ascii_letters_and_digits([100, ]) + "\n")
+        for n in range(0, p_nRows):
+            buf.append(random_digits([p_nDataLength, ]) + "\n")
         if p_nNullValueCount > 0:
             for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 100 - 1)
+                m_nPos = random.randint(0, p_nRows - 1)
                 buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.100")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "1Ks":
-        buf = []
-        for n in range(0, 1000):
-            buf.append(random_ascii_letters_and_digits([100, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 1000 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.1K")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "10Ks":
-        buf = []
-        for n in range(0, 10 * 1000):
-            buf.append(random_ascii_letters_and_digits([100, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 10*1000 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.10K")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "100Ks":
-        buf = []
-        for n in range(0, 100 * 1000):
-            buf.append(random_ascii_letters_and_digits([100, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 100*1000 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.100K")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "10n":
-        buf = []
-        for n in range(0, 10):
-            buf.append(random_digits([20, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 10 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.10")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "100n":
-        buf = []
-        for n in range(0, 100):
-            buf.append(random_digits([20, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 100 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.100")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "1Kn":
-        buf = []
-        for n in range(0, 1000):
-            buf.append(random_digits([20, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 1000 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.1K")
-        with open(seed_file, 'w') as f:
-            f.writelines(buf)
-
-    if p_szSeedName == "10Kn":
-        buf = []
-        for n in range(0, 10 * 1000):
-            buf.append(random_digits([20, ]) + "\n")
-        if p_nNullValueCount > 0:
-            for n in range(0, p_nNullValueCount):
-                m_nPos = random.randint(0, 10*1000 - 1)
-                buf[m_nPos] = "\n"
-        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.10K")
+        seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", p_szSeedName + ".seed")
         with open(seed_file, 'w') as f:
             f.writelines(buf)
 
@@ -144,95 +71,15 @@ def Load_SeedCacheFile():
     if "SQLCLI_HOME" not in os.environ:
         raise SQLCliException("Missed SQLCLI_HOME, please set it first. Seed file will created in $SQLCLI_HOME/data")
 
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.10")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["10s"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.100")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["100s"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.1K")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["1Ks"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.10K")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["10Ks"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_string.100K")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["100Ks"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.10")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["10n"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.100")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["100n"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.1K")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["1Kn"] = m_seedlines
-
-    seed_file = os.path.join(os.environ["SQLCLI_HOME"], "data", "seed_int.10K")
-    if not os.path.exists(seed_file):
-        raise SQLCliException("Seed file [" + seed_file + "] not found. Please check it.")
-    with open(seed_file, 'r') as f:
-        m_seedlines = f.readlines()
-    for m_nPos in range(0, len(m_seedlines)):
-        if m_seedlines[m_nPos].endswith("\n"):
-            m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]
-    seed_cache["10Kn"] = m_seedlines
+    m_seedpath = os.path.join(os.environ["SQLCLI_HOME"], "data", "*.seed")
+    for seed_file in glob(m_seedpath):
+        m_seedName = os.path.basename(seed_file)[:-5]            # 去掉.seed的后缀
+        with open(seed_file, 'r') as f:
+            m_seedlines = f.readlines()
+        for m_nPos in range(0, len(m_seedlines)):
+            if m_seedlines[m_nPos].endswith("\n"):
+                m_seedlines[m_nPos] = m_seedlines[m_nPos][:-1]   # 去掉回车换行
+        seed_cache[m_seedName] = m_seedlines
 
 
 # 返回随机的Boolean类型
@@ -248,33 +95,20 @@ def random_timestamp(p_arg):
     return (random.random() * (etime - stime) + stime).strftime(frmt)
 
 
-# 返回一个随机的时间   start < 返回值 < end
-def random_date(p_arg):
-    frmt = "%Y-%m-%d"
-    stime = datetime.datetime.strptime(str(p_arg[0]), frmt)
-    etime = datetime.datetime.strptime(str(p_arg[1]), frmt)
-    return (random.random() * (etime - stime) + stime).strftime(frmt)
-
-
 # 第一个参数是seed的名字
 # 第二个参数是截取的最大长度
 def random_from_seed(p_arg):
     m_SeedName = str(p_arg[0])
     m_nMaxLength = int(p_arg[1])
-    if m_SeedName.upper().endswith('s'):
-        # 随机字符串，不支持超过100位的字符串
-        if m_nMaxLength > 100:
-            m_nMaxLength = 100
-    if m_SeedName.upper().endswith('n'):
-        # 随机数字，不支持超过20位的数字
-        if m_nMaxLength > 20:
-            m_nMaxLength = 20
 
+    # 如果还没有加载种子，就先加载
+    n = len(seed_cache)
+    if n == 0:
+        Load_SeedCacheFile()
+
+    # 在种子中查找需要的内容
     if m_SeedName in seed_cache:
         n = len(seed_cache[m_SeedName])
-        if n == 0:
-            Load_SeedCacheFile()
-            n = len(seed_cache[m_SeedName])
         m_RandomRow = seed_cache[m_SeedName][random.randint(0, n - 1)]
         if len(m_RandomRow) < m_nMaxLength:
             m_RandomResult = m_RandomRow
@@ -284,6 +118,22 @@ def random_from_seed(p_arg):
     else:
         raise SQLCliException("Unknown seed [" + str(p_arg[0]) + "].  Please create it first.")
 
+# 返回一个随机的时间
+# 有3个参数：
+#   stime   开始时间
+#   etime   结束时间
+#   frmt    日期格式，可以忽略，默认为"%Y-%m-%d"
+def random_date(p_arg):
+    if len(p_arg) > 2:
+        frmt = str(p_arg[2])
+    else:
+        frmt = "%Y-%m-%d"
+    try:
+        stime = datetime.datetime.strptime(str(p_arg[0]), frmt)
+        etime = datetime.datetime.strptime(str(p_arg[1]), frmt)
+    except ValueError as ve:
+        raise SQLCliException("Invalid date format [" + str(frmt) + "] for [" + str(p_arg[0]) + "]")
+    return (random.random() * (etime - stime) + stime).strftime(frmt)
 
 def random_digits(p_arg):
     n = int(p_arg[0])
@@ -443,9 +293,25 @@ def Create_file(p_filetype, p_filename, p_formula_str, p_rows, p_options):
         m_row_struct = parse_formula_str(p_formula_str)
         buf = []
         if p_filetype.upper() == "HDFS":              # 处理HDFS文件写入
-            with m_HDFS_Handler.write(m_filename) as m_output:
-                for i in range(0, p_rows):
-                    m_output.write((get_final_string(m_row_struct) + "\n").encode())
+            # 总是覆盖服务器上的文件
+            # 每10W行提交一次服务器文件, 以避免内存的OOM问题
+            if p_rows < 100000:
+                with m_HDFS_Handler.write(hdfs_path=m_filename, overwrite=True) as m_output:
+                    for i in range(0, p_rows):
+                        m_output.write((get_final_string(m_row_struct) + "\n").encode())
+            else:
+                for i in range(0, p_rows // 100000):
+                    if i == 0:
+                        with m_HDFS_Handler.write(hdfs_path=m_filename, overwrite=True) as m_output:
+                            for j in range(0, 100000):
+                                m_output.write((get_final_string(m_row_struct) + "\n").encode())
+                    else:
+                        with m_HDFS_Handler.write(hdfs_path=m_filename, append=True) as m_output:
+                            for j in range(0, 100000):
+                                m_output.write((get_final_string(m_row_struct) + "\n").encode())
+                with m_HDFS_Handler.write(hdfs_path=m_filename, append=True) as m_output:
+                    for i in range(0, p_rows % 100000):
+                        m_output.write((get_final_string(m_row_struct) + "\n").encode())
         elif p_filetype.upper() == "KAFKA":           # 处理Kafka数据写入
             for i in range(0, p_rows):
                 m_producer.send(m_topicname, get_final_string(m_row_struct).encode())
@@ -541,21 +407,36 @@ def Convert_file(p_srcfileType, p_srcfilename, p_dstfileType, p_dstfilename):
                         break
                     m_writer.write(m_Contents)
 
+        # 对于HDFS的写入，每80M提交一次，以避免内存的OOM问题
         if m_srcFSType in ('MEM', 'FS') and m_dstFSType == "HDFS":
-            with m_srcFS.openbin(m_srcFileName, "r") as m_reader, m_dstFS.write(m_dstFileName) as m_writer:
+            bHeaderWrite = True
+            with m_srcFS.openbin(m_srcFileName, "r") as m_reader:
                 while True:
-                    m_Contents = m_reader.read(8192)
+                    m_Contents = m_reader.read(8192*10240)
                     if len(m_Contents) == 0:
                         break
-                    m_writer.write(m_Contents)
+                    if bHeaderWrite:
+                        with m_dstFS.write(m_dstFileName, overwrite=True) as m_writer:
+                            m_writer.write(m_Contents)
+                        bHeaderWrite = False
+                    else:
+                        with m_dstFS.write(m_dstFileName, append=True) as m_writer:
+                            m_writer.write(m_Contents)
 
         if m_srcFSType == "HDFS" and m_dstFSType == "HDFS":
-            with m_srcFS.read(m_srcFileName) as m_reader, m_dstFS.write(m_dstFileName) as m_writer:
+            bHeaderWrite = True
+            with m_srcFS.read(m_srcFileName) as m_reader:
                 while True:
-                    m_Contents = m_reader.read(8192)
+                    m_Contents = m_reader.read(8192*10240)
                     if len(m_Contents) == 0:
                         break
-                    m_writer.write(m_Contents)
+                    if bHeaderWrite:
+                        with m_dstFS.write(m_dstFileName, overwrite=True) as m_writer:
+                            m_writer.write(m_Contents)
+                        bHeaderWrite = False
+                    else:
+                        with m_dstFS.write(m_dstFileName, append=True) as m_writer:
+                            m_writer.write(m_Contents)
     except HdfsError as he:
         # HDFS 会打印整个堆栈信息，所以这里默认只打印第一行的信息
         if "SQLCLI_DEBUG" in os.environ:
